package at.krixec.rosary


// Class to deal with the differences in local rosaries
data class Language(
    val name: String,
    val flag: String,

    // Concerning the first 3 Ave Marias
    val aveMariaTextAbove: Boolean,
    val aveMariaTextInside: Boolean,

    // Concerning Ave Marias in the decades
    val holyMysteriesAbove: Boolean,
    val holyMysteriesInside: Boolean
)

fun getNextLang( currentLang: String ): String {

    return when( currentLang ) {
        "lt" -> "de"
        "de" -> "en"
        "en" -> "sl"
        "sl" -> "fr"
        "fr" -> "lt"
        // your language here
        else -> "lt"
    }
}

fun getAvailableLanguages() = listOf( "lt", "de", "en", "sl", "fr" )

fun getLanguage( name: String ): Language {

    return when ( name ) {
        "de" -> Language(
            name,
            flag = "\uD83C\uDDE6\uD83C\uDDF9",
            aveMariaTextAbove   = false,
            aveMariaTextInside  = true,
            holyMysteriesAbove  = false,
            holyMysteriesInside = true
        )
        "sl" -> Language(
            name,
            flag = "\uD83C\uDDF8\uD83C\uDDEE",
            aveMariaTextAbove   = false,
            aveMariaTextInside  = true,
            holyMysteriesAbove  = false,
            holyMysteriesInside = true
        )
        "fr" -> Language(
            name,
            flag = "\uD83C\uDDEB\uD83C\uDDF7",
            aveMariaTextAbove   = true,
            aveMariaTextInside  = false,
            holyMysteriesAbove  = true,
            holyMysteriesInside = false
        )
        "en" -> Language(
            name,
            flag = "\uD83C\uDDEC\uD83C\uDDE7",
            aveMariaTextAbove   = false,
            aveMariaTextInside  = false,
            holyMysteriesAbove  = true,
            holyMysteriesInside = false
        )
        "lt" -> Language(
            name,
            flag = "\uD83C\uDDFB\uD83C\uDDE6",
            aveMariaTextAbove   = false,
            aveMariaTextInside  = false,
            holyMysteriesAbove  = true,
            holyMysteriesInside = false
        )

        // The latin way
        else -> Language(
            name,
            flag = "\uD83C\uDFF4\u200D☠\uFE0F",
            aveMariaTextAbove   = false,
            aveMariaTextInside  = false,
            holyMysteriesAbove  = true,
            holyMysteriesInside = false
        )
    }
}
