package at.krixec.rosary

import android.app.LocaleManager
import android.content.Context
import android.os.Build
import android.os.Bundle
import android.os.LocaleList
import android.view.WindowManager
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.appcompat.app.AppCompatActivity
import androidx.appcompat.app.AppCompatDelegate
import androidx.compose.animation.core.Animatable
import androidx.compose.animation.core.EaseInOutCubic
import androidx.compose.animation.core.repeatable
import androidx.compose.animation.core.tween
import androidx.compose.foundation.Canvas
import androidx.compose.foundation.background
import androidx.compose.foundation.gestures.detectTapGestures
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.offset
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.width
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.FilledTonalButton
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.rotate
import androidx.compose.ui.geometry.Offset
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.Shadow
import androidx.compose.ui.input.pointer.pointerInput
import androidx.compose.ui.platform.LocalConfiguration
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.platform.LocalDensity
import androidx.compose.ui.platform.LocalView
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.drawText
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.rememberTextMeasurer
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.core.os.LocaleListCompat
import androidx.core.view.WindowCompat
import androidx.core.view.WindowInsetsCompat
import androidx.core.view.WindowInsetsControllerCompat
import at.krixec.rosary.ui.theme.Bead
import at.krixec.rosary.ui.theme.Orange
import at.krixec.rosary.ui.theme.RosariumTheme
import at.krixec.rosary.ui.theme.Shade
import kotlinx.coroutines.flow.first
import kotlinx.coroutines.launch
import kotlinx.coroutines.runBlocking


// Google is a joke
// https://stackoverflow.com/a/74413010
class MainActivity : AppCompatActivity() {

    private var isFirstTime = false

    override fun onCreate( savedInstanceState: Bundle? ) {
        super.onCreate( savedInstanceState )

        val insetsController =
            WindowCompat.getInsetsController(
                window,
                window.decorView
            )

        insetsController.apply {
            hide( WindowInsetsCompat.Type.statusBars() )
            hide( WindowInsetsCompat.Type.navigationBars() )
            systemBarsBehavior = WindowInsetsControllerCompat.BEHAVIOR_SHOW_TRANSIENT_BARS_BY_SWIPE
        }
        WindowCompat.setDecorFitsSystemWindows( window, false )

        // finally ;-;
        // https://stackoverflow.com/a/77559537
        if ( Build.VERSION.SDK_INT >= Build.VERSION_CODES.P ) {
            window.attributes.layoutInDisplayCutoutMode = WindowManager.LayoutParams.LAYOUT_IN_DISPLAY_CUTOUT_MODE_SHORT_EDGES
        }

        enableEdgeToEdge()
        setContent {

            val context = LocalContext.current
            val store = DataSave( context )

            // asynchronous preferences are retarded
            val immaculate = runBlocking {
                store.isImmaculate.first()
            }
            val autoPray = runBlocking { store.autoPrayState.first() }

            if ( immaculate )
                isFirstTime = true

            // I set the language to latin if it isn't any of the available ones
            val currentLang = LocalConfiguration.current.locales[ 0 ].toString().substring( 0, 2 )

            if ( currentLang !in getAvailableLanguages() ) {
                if ( Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU ) {
                    this.getSystemService( LocaleManager::class.java )
                        .applicationLocales = LocaleList.forLanguageTags( "lt" )
                } else {
                    AppCompatDelegate.setApplicationLocales(
                        LocaleListCompat.forLanguageTags( "lt" )
                    )
                }
            } else if ( isFirstTime ) {
                if ( Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU ) {
                    this.getSystemService( LocaleManager::class.java )
                        .applicationLocales = LocaleList.forLanguageTags( currentLang )
                } else {
                    AppCompatDelegate.setApplicationLocales(
                        LocaleListCompat.forLanguageTags( currentLang )
                    )
                }
            }

            RosariumTheme {
                App( isFirstTime, store, autoPray, this )
            }
        }
    }
}

@Composable
private fun App(
    immaculate: Boolean,
    store: DataSave,
    autoPrayState: String,
    outerContext: Context
) {

    val scope = rememberCoroutineScope()
    val context = LocalContext.current

    val showInfoPage  = remember { mutableStateOf( immaculate ) }
    val configuration = LocalConfiguration.current
    val screenWidth   = configuration.screenWidthDp.dp
    val screenHeight  = configuration.screenHeightDp.dp
    val density  = LocalDensity.current.density
    val progress = remember { Animatable(
        if ( immaculate ) 1.8f else 0f
    ) }
    val textMeasurer = rememberTextMeasurer()
    val autoPray = remember { AutoPray( autoPrayState, outerContext ) }

    @Composable
    fun InfoPage( showPage: Boolean, onChange: ( Boolean ) -> Unit ) {

        val autoPrayText = remember {
            mutableStateOf( getAutoPrayText( context, autoPrayState ) )
        }

        if ( showPage ) {
            Box(
                modifier = Modifier
                    .fillMaxSize()
                    .background( Shade )
                    .pointerInput( Unit ) {

                        detectTapGestures {
                            scope.launch {

                                store.setNotImmaculate()

                                onChange( false )
                                progress.animateTo(
                                    0f,
                                    animationSpec = repeatable(
                                        iterations = 1,
                                        animation = tween(
                                            durationMillis = 1200,
                                            easing = EaseInOutCubic
                                        )
                                    )
                                )
                            }
                        }
                    }
            ) {

                Column(
                    modifier = Modifier
                        .padding( top = 50.dp )
                        .fillMaxSize(),
                    horizontalAlignment = Alignment.CenterHorizontally
                ) {

                    Row {

                        Text(
                            color = Orange,
                            fontSize = 22.sp,
                            fontWeight = FontWeight.ExtraBold,
                            textAlign = TextAlign.Center,
                            text = stringResource( R.string.change_lang )
                        )
                        Text(
                            color = Orange,
                            fontSize = 22.sp,
                            fontWeight = FontWeight.ExtraBold,
                            textAlign = TextAlign.Center,
                            text = " →",
                            modifier = Modifier
                                .rotate( -20f )
                                .offset( 0.dp, -5.sp.value.dp )
                        )
                    }
                }

                Column(
                    modifier = Modifier
                        .padding( bottom = 160.dp )
                        .fillMaxSize(),
                    verticalArrangement = Arrangement
                        .spacedBy( 120.dp, Alignment.Bottom ),
                    horizontalAlignment = Alignment.CenterHorizontally
                ) {

                    Row(
                        modifier = Modifier
                            .width( screenWidth / 1.5f )
                            .padding( 0.dp, 50.dp ),
                        horizontalArrangement = Arrangement.SpaceEvenly,

                        ) {
                        FilledTonalButton(
                            onClick = {
                                autoPrayText.value = getAutoPrayText(
                                    context,
                                    autoPray.nextState()
                                )
                            }
                        ) {
                            Text(
                                text = "AutoPray",
                                fontWeight = FontWeight.ExtraBold,
                                fontSize = 18.sp
                            )
                        }
                        Text(
                            modifier = Modifier
                                .offset( 0.dp, 11.dp ),
                            color = Color.White,
                            text = autoPrayText.value,
                            fontWeight = FontWeight.ExtraBold,
                            fontSize = 20.sp
                        )
                    }

                    Text(
                        color = Orange,
                        fontSize = 22.sp,
                        fontWeight = FontWeight.ExtraBold,
                        textAlign = TextAlign.Center,
                        text = stringResource( R.string.tap_to_proceed ) + "\n►►►"
                    )

                    Text(
                        color = Orange,
                        fontSize = 22.sp,
                        fontWeight = FontWeight.ExtraBold,
                        textAlign = TextAlign.Center,
                        text = stringResource( R.string.swipe_to_go_back ) + "\n◄◄◄"
                    )
                }
            }
        }
    }

    Rosary( screenWidth, screenHeight, autoPray )
    InfoPage( showInfoPage.value, onChange = { showInfoPage.value = it } )
    SettingsMenu( screenWidth, density )

    val currentView = LocalView.current
    Button(
        modifier = Modifier
            .offset( -( 16.dp ), -( 10f.dp ) )
            .size( 86.dp ),
        contentPadding = PaddingValues( 1.dp ),
        colors = ButtonDefaults.buttonColors( Color.Transparent ),

        onClick = {
            currentView.keepScreenOn = false
            autoPray.stop()
            autoPray.inc()
            scope.launch {
                progress.animateTo(
                    1.8f,
                    animationSpec = repeatable(
                        iterations = 1,
                        animation = tween(
                            durationMillis = 1200,
                            easing = EaseInOutCubic
                        )
                    )
                )
                showInfoPage.value = true
            }
        }
    ) { }

    Canvas(
        modifier = Modifier
            .background( Color.Transparent )
            .size( screenWidth, screenHeight )
    ) {
        val vertPadding = screenWidth.value * density  / 12f
        val horiPadding = screenHeight.value * density / 35f
        drawLine(
            color = Color.Gray,
            start = Offset( vertPadding, 0f ),
            end   = Offset(
                vertPadding,
                horiPadding + 36.dp.toPx() - ( horiPadding + 40.dp.toPx() ) * progress.value ),
            strokeWidth = 16f
        )
        drawCircle(
            radius = 16.dp.toPx(),
            color = Bead,
            center = Offset(
                vertPadding,
                horiPadding + 16.dp.toPx() - ( horiPadding + 16.dp.toPx() ) * progress.value
            )
        )
        drawCircle(
            radius = 4.dp.toPx(),
            color = Color.Gray,
            center = Offset(
                vertPadding,
                horiPadding + 36.dp.toPx() - ( horiPadding + 16.dp.toPx() ) * progress.value
            )
        )
        val style = TextStyle.Default.copy(
            color = Color.LightGray,
            fontWeight = FontWeight.Bold,
            fontSize = 24.sp,
            shadow = Shadow( color = Color.Black, offset = Offset( 3f, 3f ) )
        )
        val textLayoutResult = textMeasurer.measure( "☰", style )
        drawText(
            textMeasurer = textMeasurer,
            text = "☰",
            topLeft = Offset(
                vertPadding - 2f - textLayoutResult.size.width / 2,
                horiPadding + 16.dp.toPx() - textLayoutResult.size.height / 2
                        - ( horiPadding + 16.dp.toPx() ) * progress.value
            ),
            style = style
        )
    }
}
