package at.krixec.rosary

import android.annotation.SuppressLint
import androidx.compose.animation.core.Animatable
import androidx.compose.animation.core.AnimationSpec
import androidx.compose.animation.core.EaseInOutCubic
import androidx.compose.animation.core.LinearEasing
import androidx.compose.animation.core.repeatable
import androidx.compose.animation.core.tween
import androidx.compose.foundation.Canvas
import androidx.compose.foundation.background
import androidx.compose.foundation.gestures.detectDragGestures
import androidx.compose.foundation.gestures.detectTapGestures
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.offset
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.derivedStateOf
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableFloatStateOf
import androidx.compose.runtime.mutableIntStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.geometry.Offset
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.Path
import androidx.compose.ui.graphics.PathMeasure
import androidx.compose.ui.graphics.Shadow
import androidx.compose.ui.graphics.asAndroidPath
import androidx.compose.ui.graphics.drawscope.Stroke
import androidx.compose.ui.input.pointer.pointerInput
import androidx.compose.ui.platform.LocalDensity
import androidx.compose.ui.platform.LocalView
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.drawText
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.rememberTextMeasurer
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.Dp
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import at.krixec.rosary.ui.theme.Bead
import at.krixec.rosary.ui.theme.Cross
import at.krixec.rosary.ui.theme.SelectedBead
import kotlinx.coroutines.launch


@SuppressLint( "CoroutineCreationDuringComposition" )
@Composable
fun Rosary( screenWidth: Dp, screenHeight: Dp, autoPray: AutoPray ) {

    val path = Path()
    val autoPrayPath = Path()

    var prayer by remember { mutableIntStateOf( 0 ) }
    var decade by remember { mutableIntStateOf( 0 ) }
    var bead   by remember { mutableIntStateOf( 0 ) }

    // I've bound the actual decade to an animation, little workaround
    var bonusDecade by remember { mutableIntStateOf( 0 ) }

    var running by remember { mutableStateOf( false ) }
    var next by remember { mutableStateOf( false ) }
    val scope = rememberCoroutineScope()
    val currentView = LocalView.current
    val textMeasurer = rememberTextMeasurer()
    var timeLeft by remember { mutableIntStateOf( 0 ) }
    val autoPrayProgress = remember { Animatable( if ( autoPray.active() ) 1f else -0.5f ) }

    if ( autoPray.needsAnimate() ) {
        autoPray.noAnimate()
        scope.launch {
            autoPrayProgress.animateTo(
                if ( autoPray.active() ) 1f else -0.5f,
                animationSpec = repeatable(
                    iterations = 1,
                    animation = tween(
                        durationMillis = 1200,
                        easing = EaseInOutCubic
                    )
                )
            )
        }
    }

    fun newTimer( id: Int ): AutoPrayTimer {

        return object: AutoPrayTimer(
            autoPray.prayerTime( prayer ) * 1000,
            1000L,
            id
        ) {
            override fun onTick( millisUntilFinished: Long ) {

                if ( iD != autoPray.timerID() )
                    cancel()
                else
                    timeLeft = ( millisUntilFinished / 1000 ).toInt()
            }

            override fun onFinish() {
                if ( autoPray.active() && iD == autoPray.timerID() )
                    next = true
            }
        }
    }

    var autoPrayTimer: AutoPrayTimer

    if ( !running && autoPray.active() && !autoPray.stopped() ) {
        running = true
        autoPrayTimer = newTimer( autoPray.timerID() )
        autoPrayTimer.start()
    }

    @Composable
    fun RosaryLoop() {

        var totalDrag by remember { mutableFloatStateOf( 0f ) }
        var hasMoved  = false
        var animating = false

        // left -> 0, middle -> 1, right -> 2
        var current by remember { mutableIntStateOf( 1 ) }

        fun getTargetValue( right: Boolean ): Float {
            return if ( current == 1 )
                if ( right )
                    -0.8f
                else
                    1.35f
            else
                0.2f
        }

        var fullyShown   = remember { false }
        val pathMeasure  = remember { PathMeasure() }
        val progress     = remember { Animatable( 0.2f ) }
        val progress2    = remember { Animatable( 0.2f ) }
        val animatedPath = remember {
            derivedStateOf {
                val destination = Path()
                pathMeasure.setPath( path, false )

                pathMeasure.getSegment(
                    0f,
                    pathMeasure.length + progress2.value * pathMeasure.length - pathMeasure.length * 0.2f,
                    destination
                )
                destination
            }
        }

        suspend fun moveRosary(
            to: Float,
            duration: Int,
            first: Boolean = false
        ) {

            val spec: AnimationSpec< Float > = repeatable(
                iterations = 1,
                animation = tween(
                    durationMillis = duration,
                    easing = LinearEasing
                )
            )
            if ( first )
                progress.animateTo( to, animationSpec = spec )
            else
                progress2.animateTo( to, animationSpec = spec )

        }

        fun goRight() {

            if ( !animating ) {

                if ( decade == 0 ) {

                    if ( bead < 7 ) {
                        bead += 1
                        prayer = when ( bead ) {
                            1 -> 4
                            2 -> 1
                            6 -> 2
                            7 -> 1

                            else -> -1
                        }
                    } else {

                        scope.launch {
                            animating = true
                            prayer = -1
                            bonusDecade = 1
                            moveRosary( -1f, 1500 )

                            bonusDecade = 0
                            decade += 1
                            bead   = 0
                            progress2.snapTo( -0.5f )
                            moveRosary( -1f, 1000 )

                            fullyShown = true
                            animating = false
                        }
                    }
                } else {
                    if ( decade == 5 && bead == 10 ) {

                        prayer = 5
                        bead += 1

                    } else if ( decade == 5 && bead == 11 ) {

                        autoPray.stop()
                        currentView.keepScreenOn = false
                        scope.launch {

                            animating = true
                            fullyShown = false
                            prayer = 0
                            moveRosary( 0.2f, 1500 )

                            bead   = 0
                            decade = 0
                            progress2.snapTo( -1f )
                            moveRosary( 0.2f, 1000 )

                            animating = false
                        }
                    } else if ( ( current == 1 && bead >= 10 )
                        || ( current != 1 && bead >= 4 )
                    ) {

                        scope.launch {

                            animating = true
                            if ( current != 1 ) {
                                progress.snapTo( 1.34f )
                                bead = 0
                                prayer = -1
                            } else {
                                bead = 3
                                prayer = 2
                            }

                            if ( current == 1 )
                                if ( decade < 5 )
                                    decade += 1
                                else
                                    decade = 0

                            if ( decade != 0 ) {
                                moveRosary( getTargetValue( true ), 2000, true )

                                if ( current < 2 )
                                    current += 1
                                else {
                                    current = 1
                                }
                            } else {
                                fullyShown = false
                                bead = 0
                                moveRosary( 0.2f, 1500 )
                            }
                            animating = false
                        }
                    } else {
                        bead += 1
                        if ( bead == 10 )
                            prayer = 3
                        if ( current != 1 )
                            prayer = 1
                    }
                }
            }
        }

        fun goLeft() {
            if ( decade == 0 && bead == 0 ) {
                scope.launch {

                    animating = true
                    prayer = 5
                    bonusDecade = 5
                    moveRosary( -1f, 1500 )

                    bonusDecade = 0
                    decade = 5
                    bead   = 11
                    progress2.snapTo( -0.5f )
                    moveRosary( -1f, 1000 )

                    animating  = false
                    fullyShown = true
                }
            } else if ( decade == 1 && bead == 0 ) {

                scope.launch {

                    animating  = true
                    fullyShown = false
                    prayer = 1
                    moveRosary( 0.2f, 1500 )

                    decade = 0
                    bead   = 7
                    progress2.snapTo( -1f )
                    moveRosary( 0.2f, 1000 )

                    animating = false
                }

            } else if ( ( current == 1 && bead <= 0 )
                || ( current != 1 && bead <= 3 )
            ) {
                scope.launch {
                    animating = true
                    if ( current != 1 ) {
                        progress.snapTo( -0.78f )
                        bead = 10
                        prayer = 3
                    } else {
                        bead = 4
                        prayer = 1
                    }

                    if ( current == 1 )
                        if ( decade > 0 )
                            decade -= 1
                        else {
                            decade = 5
                        }

                    moveRosary( getTargetValue( false ), 2000, true )

                    if ( current > 0 )
                        current -= 1
                    else {
                        current = 1
                    }
                    animating = false
                }
            } else {
                bead -= 1
                if ( decade != 0 )
                    prayer = -1
                else {
                    if ( bead < 7 ) {
                        prayer = when ( bead ) {
                            0 -> 0
                            1 -> 4
                            2 -> 1
                            6 -> 2
                            //7 -> 1

                            else -> -1
                        }
                    }
                }
                if ( decade == 5 && bead == 10 )
                    prayer = 3
                else if ( current != 1 && decade != 0 )
                    prayer = 2
            }
        }

        // invisible box to catch touch input
        Box(
            modifier = Modifier
                .fillMaxSize()
                .pointerInput( Unit ) {

                    detectTapGestures {

                        if ( autoPray.active() && !autoPray.stopped() ) {
                            next = true

                        } else {
                            goRight()
                        }
                    }
                }
                .pointerInput( Unit ) {
                    detectDragGestures(
                        onDragStart = {
                            totalDrag = 0f
                            hasMoved = false
                        }
                    ) { change, dragAmount ->
                        change.consume()

                        totalDrag += dragAmount.x

                        if ( !hasMoved && !animating && totalDrag > 40 ) {

                            hasMoved = true
                            if ( autoPray.active() && !autoPray.stopped() ) {
                                next = true

                            } else {
                                goRight()
                            }
                        } else if ( !hasMoved && !animating && totalDrag < -40 ) {

                            hasMoved = true

                            if ( autoPray.active() && !autoPray.stopped() ) {
                                autoPray.inc()
                                goLeft()
                                autoPrayTimer = newTimer( autoPray.timerID() )
                                autoPrayTimer.start()
                            } else
                                goLeft()

                        }
                    }
                }
        )

        val density = LocalDensity.current.density

        val screenWidthInPx  = screenWidth.value * density
        val screenHeightInPx = screenHeight.value * density

        Text(
            getPrayer( prayer, decade, bonusDecade, bead ),
            modifier = Modifier
                .fillMaxSize()
                .padding( 20.dp, 100.dp ),
            textAlign = TextAlign.Center,
            color = Color.LightGray,
            fontSize = 18.sp,
            fontWeight = FontWeight.ExtraBold
        )

        Canvas(
            modifier = Modifier
                .background( Color.Transparent )
                .fillMaxSize()
        ) {

            if ( decade != 0 ) {
                val x0 = -100f
                val y0 = screenHeightInPx / 1.2f
                val y1 = screenHeightInPx / 1.6f
                val y2 = screenHeightInPx / 0.9f
                val y3 = screenHeightInPx / 1.2f

                path.reset()
                path.moveTo( x0, if ( fullyShown ) y0 else y0 + screenHeightInPx * ( progress2.value + 1f ) )
                path.cubicTo(
                    x1 = screenWidthInPx / 2f,
                    y1 = if ( fullyShown ) y1 else y1 + screenHeightInPx / 3 * ( progress2.value + 1f ) ,
                    x2 = screenWidthInPx / 2f,
                    y2 = if ( fullyShown ) y2 else y2 + screenHeightInPx / 3 * ( progress2.value + 1f ) ,
                    x3 = screenWidthInPx + 100f,
                    y3 = if ( fullyShown ) y3 else y3 + screenHeightInPx / 3 * ( progress2.value + 1f ) ,
                )

                drawPath(
                    color = Color.Gray,
                    path  = path,
                    style = Stroke(
                        width = 8.dp.toPx()
                    )
                )

                // side pater nosters
                for ( i in 0..5 ) {

                    val pos = FloatArray( 2 )
                    val tan = FloatArray( 2 )
                    val mod = if ( i <= 2 ) -1 else 1

                    android.graphics.PathMeasure().apply {
                        setPath( path.asAndroidPath(), false )
                        getPosTan( length * ( progress.value + mod + ( i + 2 ) * 0.045f ),
                            pos,
                            tan
                        )
                    }

                    val color = if ( prayer == 1 ) SelectedBead else Bead
                    if ( i % 2 != 0 && i < 3 ) {
                        drawCircle(
                            radius = 16.dp.toPx(),
                            color  = color,
                            center = Offset( pos[ 0 ], pos[ 1 ] )
                        )
                    } else if ( i % 2 == 0 && i < 3 ) {
                        drawCircle(
                            radius = 8.dp.toPx(),
                            color  = Color.Gray,
                            center = Offset( pos[ 0 ], pos[ 1 ] )
                        )
                    } else if ( i % 2 != 0 ) {
                        drawCircle(
                            radius = 8.dp.toPx(),
                            color  = Color.Gray,
                            center = Offset( pos[ 0 ], pos[ 1 ] )
                        )
                    } else {
                        drawCircle(
                            radius = 16.dp.toPx(),
                            color  = color,
                            center = Offset( pos[ 0 ], pos[ 1 ] )
                        )
                    }
                }

                // grey knots
                for ( i in 0..1 ) {

                    val mod = if ( i == 0 ) -1f else 14.15f
                    val pos = FloatArray( 2 )
                    val tan = FloatArray( 2 )

                    android.graphics.PathMeasure().apply {
                        setPath( path.asAndroidPath(), false )
                        getPosTan( length * ( progress.value + mod * 0.048f ),
                            pos,
                            tan
                        )
                    }

                    drawCircle(
                        radius = 8.dp.toPx(),
                        color  = Color.Gray,
                        center = Offset( pos[ 0 ], pos[ 1 ] )
                    )

                }
                for ( i in 0..9 ) {

                    val pos = FloatArray( 2 )
                    val tan = FloatArray( 2 )

                    android.graphics.PathMeasure().apply {
                        setPath( path.asAndroidPath(), false )
                        getPosTan( length * ( progress.value + i * 0.07f ),
                            pos,
                            tan
                        )
                    }
                    val color = if ( bead == i && prayer == -1 ) SelectedBead else Bead
                    drawCircle(
                        radius = 16.dp.toPx(),
                        color = color,
                        center = Offset( pos[ 0 ], pos[ 1 ] )
                    )
                }
            } else {

                // You should be able to adjust this
                // in the settings at some point
                val yShifter = 100f

                val x0 = screenWidthInPx + 220f
                val y0 = screenHeightInPx / 1.8f + yShifter

                path.reset()
                path.moveTo( x0, y0 )
                path.cubicTo(
                    x1 = screenWidthInPx  / 2f,
                    y1 = screenHeightInPx / 1.6f + yShifter,
                    x2 = screenWidthInPx  / 1.4f,
                    y2 = screenHeightInPx / 1.4f + yShifter,
                    x3 = screenWidthInPx  / 1.6f,
                    y3 = screenHeightInPx / 1.3f + yShifter
                )

                drawPath(
                    color = Color.Gray,
                    path = animatedPath.value,
                    style = Stroke(
                        width = 8.dp.toPx()
                    )
                )

                for ( i in 0..5 ) {

                    val pos = FloatArray( 2 )
                    val tan = FloatArray( 2 )

                    when ( i ) {
                        0 -> {

                            android.graphics.PathMeasure().apply {
                                setPath( path.asAndroidPath(), false )
                                getPosTan(
                                    length * ( progress2.value + 0.8f ),
                                    pos,
                                    tan
                                )
                            }

                            val color = Cross
                            val girth = 24.dp.toPx()
                            drawLine(
                                color = color,
                                start = Offset( pos[ 0 ] - 45.dp.toPx() - girth / 2, pos[ 1 ] ),
                                end   = Offset( pos[ 0 ] + 37.5.dp.toPx() - girth / 2, pos[ 1 ] + 48.7.dp.toPx() ),
                                strokeWidth = girth
                            )
                            drawLine(
                                color = color,
                                start = Offset( pos[ 0 ], pos[ 1 ] - 3.75.dp.toPx() ),
                                end   = Offset( pos[ 0 ] - 67.4.dp.toPx(), pos[ 1 ] + 112.36.dp.toPx() ),
                                strokeWidth = girth
                            )
                        }
                        1 -> {
                            // if the following numbers seem arbitrary, that's because they are
                            for ( j in -1..1 ) {

                                android.graphics.PathMeasure().apply {
                                    setPath( path.asAndroidPath(), false )
                                    getPosTan( length * ( progress2.value + j * 0.076f + 0.13f ),
                                        pos,
                                        tan
                                    )
                                }

                                val color = if ( bead == 7 ) SelectedBead else Bead
                                if ( j != 0 )
                                    drawCircle(
                                        radius = 8.dp.toPx(),
                                        color  = Color.Gray,
                                        center = Offset( pos[ 0 ], pos[ 1 ] )
                                    )
                                else
                                    drawCircle(
                                        radius = 16.dp.toPx(),
                                        color  = color,
                                        center = Offset( pos[ 0 ], pos[ 1 ] )
                                    )
                            }
                            android.graphics.PathMeasure().apply {
                                setPath( path.asAndroidPath(), false )
                                getPosTan( length * ( progress2.value + 0.01f ),
                                    pos,
                                    tan
                                )
                            }

                        }
                        in 2..4 -> {
                            android.graphics.PathMeasure().apply {
                                setPath( path.asAndroidPath(), false )
                                getPosTan( length * ( progress2.value + i * 0.105f + 0.08f ),
                                    pos,
                                    tan
                                )
                            }

                            val color = if ( bead == 7 - i ) SelectedBead else Bead
                            drawCircle(
                                radius = 16.dp.toPx(),
                                color  = color,
                                center = Offset(pos[ 0 ], pos[ 1 ])
                            )
                        }
                        else -> {
                            for ( j in -1..1 ) {

                                android.graphics.PathMeasure().apply {
                                    setPath( path.asAndroidPath(), false )
                                    getPosTan(
                                        length * ( progress2.value + i * 0.125f + j * 0.077f + 0.035f ),
                                        pos,
                                        tan
                                    )
                                }

                                val color = if ( bead == 2 ) SelectedBead else Bead
                                if ( j != 0 )
                                    drawCircle(
                                        radius = 8.dp.toPx(),
                                        color  = Color.Gray,
                                        center = Offset( pos[ 0 ], pos[ 1 ] )
                                    )
                                else
                                    drawCircle(
                                        radius = 16.dp.toPx(),
                                        color  = color,
                                        center = Offset( pos[ 0 ], pos[ 1 ] )
                                    )
                            }
                        }
                    }
                }
            }
        }

        if ( next ) {
            next = false
            autoPray.inc()
            goRight()

            if ( !autoPray.stopped() ) {

                autoPrayTimer = newTimer( autoPray.timerID() )
                autoPrayTimer.start()
            }
        }
    }

    @Composable
    fun AutoPrayButton() {

        var symbol by remember { mutableStateOf( "▶" ) }
        symbol = getSymbol( autoPray.stopped() )

        Box(
            modifier = Modifier
                .fillMaxHeight()
        ) {

            Button(
                modifier = Modifier
                    .size( 100.dp )
                    .offset( 0.dp, ( -10 ).dp)
                    .align( Alignment.BottomStart ),
                colors = ButtonDefaults.buttonColors(
                    containerColor = Color.Transparent,
                    disabledContainerColor = Color.Transparent
                ),
                enabled = autoPray.active(),
                onClick = {
                    if ( autoPray.stopped() ) {
                        currentView.keepScreenOn = true
                        symbol = "■"
                        autoPray.inc()
                        autoPrayTimer = newTimer( autoPray.timerID() )
                        autoPrayTimer.start()
                        autoPray.pray()
                    } else {
                        currentView.keepScreenOn = false
                        symbol = "▶"
                        autoPray.stop()
                        autoPray.inc()
                    }
                }
            ) { }

            Canvas(
                modifier = Modifier
                    .size( screenWidth / 3f, screenHeight / 4f )
                    .align( Alignment.BottomStart )
            ) {
                val screenWidthInPx  = screenWidth.value * density
                val screenHeightInPx = screenHeight.value * density
                val fixedScreenWidthInPx  = screenWidthInPx / 3f
                val fixedScreenHeightInPx = screenHeightInPx / 4f

                val x0 = -10f
                val y0 = fixedScreenHeightInPx / 2.2f
                val x1 = fixedScreenWidthInPx  / 1.8f
                val y1 = fixedScreenHeightInPx / 1.5f
                val x2 = -20f
                val y2 = fixedScreenHeightInPx / 1.2f
                val x3 = fixedScreenWidthInPx / 2.5f


                autoPrayPath.reset()
                autoPrayPath.moveTo( x0, y0 )
                autoPrayPath.cubicTo(
                    x1 = x1 * autoPrayProgress.value,
                    y1 = y1,
                    x2 = x2 * autoPrayProgress.value,
                    y2 = y2,
                    x3 = x3 * autoPrayProgress.value,
                    y3 = fixedScreenHeightInPx + 10f
                )

                drawPath(
                    color = Color.Gray,
                    path = autoPrayPath,
                    style = Stroke(
                        width = 8.dp.toPx()
                    )
                )

                val style  = TextStyle.Default.copy(
                    color = Color.LightGray,
                    fontSize = 28.sp,
                    shadow = Shadow( color = Color.Black, offset = Offset( 4f, 4f ) )
                )
                val numbersStyle = TextStyle.Default.copy(
                    color = Color.Gray,
                    fontSize = 20.sp
                )
                val textLayoutResult = textMeasurer.measure( symbol, style )

                for ( i in -1..1 ) {
                    val pos = FloatArray( 2 )
                    val tan = FloatArray( 2 )

                    android.graphics.PathMeasure().apply {
                        setPath( autoPrayPath.asAndroidPath(), false )
                        getPosTan(
                            length * ( 0.48f + 0.185f * i ),
                            pos,
                            tan
                        )
                    }

                    if ( i == 0 )
                        drawCircle(
                            radius  = 16.dp.toPx(),
                            color   = Bead,
                            center = Offset( pos[ 0 ], pos[ 1 ] )
                        )
                    else
                        drawCircle(
                            radius = 8.dp.toPx(),
                            color  = Color.Gray,
                            center  = Offset( pos[ 0 ], pos[ 1 ] )
                        )
                }

                val pos = FloatArray( 2 )
                val tan = FloatArray( 2 )

                android.graphics.PathMeasure().apply {
                    setPath( autoPrayPath.asAndroidPath(), false )
                    getPosTan(
                        length * 0.5f,
                        pos,
                        tan
                    )
                }

                drawText(
                    textMeasurer = textMeasurer,
                    text    = symbol,
                    style   = style,
                    topLeft = Offset(
                        pos[ 0 ] - textLayoutResult.size.width / 2f + 4f,
                        pos[ 1 ] - textLayoutResult.size.height / 2f - 14f
                    )
                )

                if ( !autoPray.stopped() )
                    drawText(
                        textMeasurer = textMeasurer,
                        text = ( timeLeft + 1 ).toString(),
                        style = numbersStyle,
                        topLeft = Offset(
                            pos[ 0 ] + textLayoutResult.size.width,
                            pos[ 1 ] - textLayoutResult.size.height
                        )
                    )
            }
        }
    }

    RosaryLoop()
    AutoPrayButton()
}
