package at.krixec.rosary

import android.app.LocaleManager
import android.graphics.PathMeasure
import android.os.Build
import android.os.LocaleList
import androidx.appcompat.app.AppCompatDelegate
import androidx.compose.animation.core.Animatable
import androidx.compose.animation.core.LinearEasing
import androidx.compose.animation.core.repeatable
import androidx.compose.animation.core.tween
import androidx.compose.foundation.Canvas
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.offset
import androidx.compose.foundation.layout.size
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults
import androidx.compose.runtime.Composable
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.ui.Modifier
import androidx.compose.ui.geometry.Offset
import androidx.compose.ui.geometry.Rect
import androidx.compose.ui.geometry.Size
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.Path
import androidx.compose.ui.graphics.Shadow
import androidx.compose.ui.graphics.asAndroidPath
import androidx.compose.ui.graphics.drawscope.Stroke
import androidx.compose.ui.platform.LocalConfiguration
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.drawText
import androidx.compose.ui.text.rememberTextMeasurer
import androidx.compose.ui.unit.Dp
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.core.os.LocaleListCompat
import at.krixec.rosary.ui.theme.Bead
import kotlinx.coroutines.launch


@Composable
fun SettingsMenu( screenWidth: Dp, density: Float ) {

    val circle = Path()
    val scope = rememberCoroutineScope()
    val context = LocalContext.current
    val textMeasurer = rememberTextMeasurer()
    var currentLang = LocalConfiguration.current.locales[ 0 ].toString()
    var langDetails = getLanguage( currentLang )

    val screenWidthInPx = screenWidth.value * density

    val progress  = remember { Animatable( 0f ) }
    var animating = remember { false }
    val radius = screenWidthInPx / 2.8f
    val circleCenter: Pair< Float, Float >

    circle.reset()
    circle.addOval(
        Rect(
            Offset( screenWidthInPx - radius / 2, 0f - radius / 2 ),
            Size( radius, radius / 1.2f )
        )
    )


    val flagPos = FloatArray( 2 )
    val flagTan = FloatArray( 2 )
    PathMeasure().apply {
        setPath( circle.asAndroidPath(), false )
        getPosTan(
            length * ( 2 * ( 1f / 3f ) - 0.03f ),
            flagPos,
            flagTan
        )
    }
    circleCenter = Pair( flagPos[ 0 ], flagPos[ 1 ] )

    Canvas(
        modifier = Modifier
            .background( Color.Transparent )
            .fillMaxSize()
    ) {
        drawPath(
            circle,
            style = Stroke( 16f ),
            color = Color.Gray
        )

        for ( bead in 0..2 ) {

            for ( knot in -1..1 ) {
                val pos = FloatArray( 2 )
                val tan = FloatArray( 2 )

                PathMeasure().apply {
                    setPath(circle.asAndroidPath(), false)
                    getPosTan(
                        length * ( progress.value + ( bead + ( knot * 0.17f ) ) * ( 1f / 3f ) - 0.03f ),
                        pos,
                        tan
                    )
                }

                if ( knot == 0 ) {
                    drawCircle(
                        radius = 16.dp.toPx(),
                        color = Bead,
                        center = Offset( pos[ 0 ], pos[ 1 ] )
                    )

                } else {
                    drawCircle(
                        radius = 8.dp.toPx(),
                        color = Color.Gray,
                        center = Offset( pos[0], pos[1] )
                    )
                }
            }
        }

        val flag = if ( !animating ) langDetails.flag else ""
        val style = TextStyle.Default.copy(
            fontSize = 20.sp,
            shadow = Shadow( color = Color.Black, offset = Offset( 5f, 8f ) )
        )
        val textLayoutResult = textMeasurer.measure( flag, style )

        drawText(
            textMeasurer = textMeasurer,
            text = flag,
            style = style,
            topLeft = Offset(
                circleCenter.first - textLayoutResult.size.width / 2 - 4,
                circleCenter.second - textLayoutResult.size.height / 2 - 4
            )
        )
    }

    val pos = FloatArray( 2 )
    val tan = FloatArray( 2 )

    PathMeasure().apply {
        setPath( circle.asAndroidPath(), false )
        getPosTan( length * 0.55f,
            pos,
            tan
        )
    }

    Button(
        modifier = Modifier
            .offset( ( circleCenter.first / density ).dp - 43.dp, ( circleCenter.second / density ).dp - 43.dp )
            .size( 86.dp ),
        contentPadding = PaddingValues( 1.dp ),

        onClick = {
            scope.launch {
                if ( !animating ) {
                    animating = true
                    progress.animateTo(
                        progress.value + ( 1f / 3f ),
                        animationSpec = repeatable(
                            iterations = 1,
                            animation = tween(
                                durationMillis = 1000,
                                easing = LinearEasing
                            )
                        )
                    )

                    progress.snapTo( 0f )

                    val lang = getNextLang( currentLang )

                    // needed to not have a delay
                    currentLang = lang
                    langDetails = getLanguage( currentLang )

                    // locales in Android give me turbo cancer
                    if ( Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU ) {
                        context.getSystemService( LocaleManager::class.java )
                            .applicationLocales = LocaleList.forLanguageTags( lang )
                    } else {
                        AppCompatDelegate.setApplicationLocales(
                            LocaleListCompat.forLanguageTags(
                                lang
                            )
                        )
                    }

                    animating = false
                }
            }
        },
        colors = ButtonDefaults.buttonColors(
            containerColor = Color.Transparent
        )
    ) { }
}
